@@ -24,10 +24,7 @@
 """
 
 import unittest
-try:
-    from unittest import mock
-except ImportError:
-    import mock
+from unittest import mock
 from datetime import datetime, timedelta
 
 import numpy as np
